#!/usr/bin/python2.7

from twisted.internet import reactor
from txosc import async, dispatch, osc
import sys
import threading
import time

host_multicast = "224.0.0.11"
port_multicast = 18888
host_local = "127.0.0.1"
port_out = 8000
port_in = 9000

t0 = time.time()
datas_dict = {}

def send_with_multicast_handler(message, address):
    # send the same message to multicast
    multicast_sender.send_message(message)
    # only to debug
    global t0
    if time.time() - t0 > 2:
        t0 = time.time()
        print("Relay is running %s" % datas_dict)

class UDPReceiver(object):
    def __init__(self, port):
        self.port = port
        self.receiver = dispatch.Receiver()
        self._server_port = reactor.listenUDP(self.port, async.DatagramServerProtocol(self.receiver))
        print("Listening udp on: %s : %s" % (host_local, self.port))
        # handler for all usefull data from a player
        self.receiver.addCallback("/player_data", send_with_multicast_handler)
        # quit
        #self.receiver.addCallback("/quit", self.quit_handler)

    def quit_handler(self, message, address):
        print("Quit : Got %s from %s" % (message, address))
        reactor.stop()
        print("Goodbye.")

class UDPMulticastSender(object):
    def __init__(self, port, host=host_multicast):
        self.port = port
        self.host = host
        self.client = async.DatagramClientProtocol()
        self._client_port = reactor.listenUDP(0, self.client)
        print("Sending multicast on: %s : %s" % (self.host, self.port))

    def send_message(self, message):
        self.client.send(message, (self.host, self.port))

def send_with_udp_handler(message, address):
    udp_sender.send_message(message)

def send_bundle_with_udp_handler_1(message, address):
    # recreate dict
    global datas_dict
    datas_dict[address] = message.getValues()
    # Recreate bundle
    bundle = osc.Bundle()
    for key, value in datas_dict.items():
        element = osc.Message("/player/1")
        for data in value:
            element.add(data)
        bundle.add(element)
    udp_sender.send_message(bundle)

def send_bundle_with_udp_handler_2(message, address):
    # recreate dict
    global datas_dict
    datas_dict[address] = message.getValues()
    # Recreate bundle
    bundle = osc.Bundle()
    for key, value in datas_dict.items():
        element = osc.Message("/player/2")
        for data in value:
            element.add(data)
        bundle.add(element)
    udp_sender.send_message(bundle)

class UDPMulticastReceiver(object):
    def __init__(self, port):
        self.port = port
        self.receiver = dispatch.Receiver()
        self._server_port = reactor.listenMulticast(self.port, async.MulticastDatagramServerProtocol(self.receiver, host_multicast), listenMultiple=True)
        print("Listening multicast on: %s : %s" % (host_multicast, self.port))
        self.receiver.addCallback("/master", send_with_udp_handler)
        self.receiver.addCallback("/player/1", send_bundle_with_udp_handler_1)
        self.receiver.addCallback("/player/2", send_bundle_with_udp_handler_2)

class UDPSender(object):
    def __init__(self, port, host=host_local):
        self.port = port
        self.host = host
        self.client = async.DatagramClientProtocol()
        self._client_port = reactor.listenUDP(0, self.client)
        print("Sending multicast on: %s : %s" % (self.host, self.port))

    def send_message(self, message):
        self.client.send(message, (self.host, self.port))

if __name__ == "__main__":
    # Set instance
    udp_receiver = UDPReceiver(port_in)
    udp_sender = UDPSender(port_out)
    multicast_sender = UDPMulticastSender(port_multicast)
    multicast_reicever = UDPMulticastReceiver(port_multicast)
    reactor.run()



