'''
# Copyright (C) Labomedia April 2012
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#############################################################################
'''

import GameLogic as gl
import socket
import OSC3.OSC as OSC

# init global var
ip = gl.ip
port = gl.port_in
buffer_size = 1024

stack =[] # player order for display in blender, a little bit complexed, but not complicated
data = [] # osc data with modif along the script
players_data = [] # list of list
master = 0 # item in players_data who master = 1, gl.master=true if I'm master
# master and gl.master are not the same variable !!

# Get controller and owner
controller = gl.getCurrentController()
owner = controller.owner
# Get all scenes
scenes = gl.getSceneList()
# For all scenes, all objets, all prop
# Get bat and ball
for scen in scenes:
    for obj in scen.objects:
        if str(obj) == "2_bat_1" :
            bat_1 = obj
        if str(obj) == "ball2":
            ball2 = obj

def main():
    data = get_osc()
    #data=['/for_all_players',',siiiffffsiiiffff','10.0.0.5',0,8,0,7,1,5,-1 ,
    #                                             '10.0.0.4',1,9,10,9,0,0,0 ]

    players_data = set_players_data_with_data(data)
    # players_data = [ ['10.0.0.5',0,8,0,7,1,5,1], 8 items
    #                  ['10.0.0.4',1,9,10,9,0,8,0] ]
    #            ip,master,number,score,batx,baty,ballx,bally

    # stack is complicated
    stack, master = create_players_stack(players_data)

    print_usefull_var((players_data, stack, master))
    # [['10.0.0.5', 0, 8, 0, 7, 1, 5, -1], ['10.0.0.4', 1, 9, 10, -9, -0, -2, -0]], [9, 8], 0)

    if not gl.master:
        set_ball_position(players_data)
    set_bat_position(players_data)

def set_bat_position(players_data):
    if players_data:
        bat_1.localPosition = [players_data[0][4], players_data[0][5], 1]

def set_ball_position(players_data):
    if players_data:
        ball2.localPosition = [players_data[0][6], players_data[0][7], 1]

def get_osc():
    global data
    # Connect Blender
    if not gl.connected :
        gl.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        try:
            gl.sock.bind((ip, port))
            gl.sock.setblocking(0)
            gl.sock.settimeout(0.005)
            gl.sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, buffer_size)
            print('Get OSC on IP={} Port={} Buffer={}'.format(ip, port, buffer_size))
            gl.connected = True
        except:
            print('No connected to IP={} Port={}'.format(ip, port))
            pass

    # If Blender connected
    if gl.connected :
        try:
            raw_data = gl.sock.recv(buffer_size)
            #print(raw_data)
            data = OSC.decodeOSC(raw_data)
            #print(data)
        except socket.error:
            pass
    return data

def chunks(l, n):
    # coupe en n listes de liste de len l
    return [l[i:i+n] for i in range(0, len(l), n)]

def set_players_data_with_data(data):
    global players_data
    if '/for_all_players' in data :
        data = data[2:]
        if len(data) % 8 == 0 : # verif pour qu il ne manque pas de data
            players_data = chunks(data, 8) # division en sous liste
    return players_data

def create_players_stack(players_data):
    player_list = []
    global master
    global stack
    for i in range(len(players_data)):
        player_list.append(players_data[i][2])
        if players_data[i][1] == 1:
            master = i #  master position of the list in the global list
        if players_data[i][0] == gl.local_ip: # where is my ip
            gl.player = players_data[i][2] # this is my player number
            # suis-je master ou slave ?
            if players_data[i][1] == 1:
                gl.master = True
            else :
                gl.master = False # I am not master, set ball position
    # je trie en ordre croissant [5, 1, 9, 6]
    if len(player_list) > 1:
        player_list.sort()
        ix = player_list.index(gl.player)
        a = player_list[ix:]
        b = player_list[:ix]
        stack = a + b
    return stack, master

def print_usefull_var(usefull_var):
    # print every 60 frames
    gl.tp += 1
    if gl.tp == 60:
        print(usefull_var)
        gl.tp = 0

def i_quit(data):
    # very bad atchtung
    global players_data
    if '/for_all_players' in data :
        owner["quit"] = True
        gl.sock.close()

main()

##([['10.0.0.5', 0, 2, 7, -9.447389602661133, 3.700000286102295, 3.194204330444336, 2.126478433609009],
##['10.0.0.4', 1, 1, 10, -9.447389602661133, -0.0056021548807621, 3.7317984104156494, 4.181915760040283]],
##[1, 2], 1)
